#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import ntpath
import subprocess
from threading import Thread, Event
import random
import logging
import time
import psutil
import struct
import traceback
import warnings

try:
    from scapy.all import wrpcap, conf, ETH_P_ALL, sniff, Scapy_Exception
except ImportError:
	print('[-]: scapy is not installed, please install it by running: pip3 install scapy')
	exit(2)

from .android import Android
 
INVALID_IPV4 = "0.0.0.0"
INVALID_IPV6 = "::"

# Configure logging to suppress scapy warnings
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
warnings.simplefilter("ignore", ResourceWarning)


def terminate_lingering_processes(parent_pid):
    parent = psutil.Process(parent_pid)
    for child in parent.children(recursive=True):
        print(f"Terminating child process: {child.pid} ({child.name()})")
        child.terminate()
        try:
            child.wait(timeout=2)
        except psutil.TimeoutExpired:
            print(f"Forcing kill of child process: {child.pid}")
            child.kill()

class PCAP:
    
    def __init__(self,pcap_file_name,SSL_READ,SSL_WRITE, doFullCapture, isMobile, print_debug_infos=False):
        self.pcap_file_name = pcap_file_name
        if isMobile is True:  # No device ID provided
            self.device_id = None
        else:
            self.device_id = isMobile
        self.pkt ={}
        self.print_debug_infos = print_debug_infos
        
            
        self.is_Mobile = isMobile
        
        # ssl_session[<SSL_SESSION id>] = (<bytes sent by client>,
        #                                  <bytes sent by server>)
        self.ssl_sessions = {}
        self.SSL_READ = SSL_READ
        self.SSL_WRITE = SSL_WRITE
        
        if doFullCapture:
            if isMobile:
                self.android_Instance = Android(self.print_debug_infos,device_id=self.device_id)
            self.full_capture_thread = self.get_instance_of_FullCaptureThread()
            self.full_capture_thread.start()
            if self.full_capture_thread.is_alive():
                print("[*] capturing whole traffic of target app")
        else:
            print("[*] capturing only plaintext data")
            self.pcap_file = self.__create_plaintext_pcap()
            
    

    def get_instance_of_FullCaptureThread(self):
        
        pcap_class = self
        
        class FullCaptureThread(Thread):
            
            def __init__(self):
                super(FullCaptureThread,self).__init__()
                self.pcap_file_name = pcap_class.pcap_file_name
                self.daemon = True
                self.socket = None
                self.stop_capture = Event()
                self.tmp_pcap_name = self._get_tmp_pcap_name()
                
                self.mobile_subprocess = -1
                self.android_capture_process = -1    
                self.is_Mobile = pcap_class.is_Mobile
                
            
            def _get_pcap_base_name(self):
                head, tail = ntpath.split(self.pcap_file_name)
                return tail or ntpath.basename(head)
                
            
            def _get_pcap_dir_path(self):
                dirname_wihtout_last_delimiter = ntpath.dirname(self.pcap_file_name)
                if len(dirname_wihtout_last_delimiter) > 1:
                    return dirname_wihtout_last_delimiter + self.pcap_file_name[len(dirname_wihtout_last_delimiter):(len(dirname_wihtout_last_delimiter)+1)]
                else:
                    return dirname_wihtout_last_delimiter
            
            
            def _get_tmp_pcap_name(self):
                return self._get_pcap_dir_path()+"_"+self._get_pcap_base_name()
                
            
            def write_packet_to_pcap(self,packet):
                wrpcap(self.tmp_pcap_name, packet, append=True)  #appends packet to output file
            
            
            def clean_up_and_exit(self):
                """Gracefully exit the FullCaptureThread"""
                print("[*] Cleaning up FullCaptureThread resources.")
                if self.socket:
                    try:
                        print("[*] Closing network socket.")
                        self.socket.close()
                    except Exception as e:
                        print(f"[-] Error while closing the socket: {e}")
                if self.android_capture_process != -1:
                    try:
                        print("[*] Terminating android capture process.")
                        self.android_capture_process.terminate()
                        self.android_capture_process.wait(timeout=2)
                    except Exception as e:
                        print(f"[-] Error while terminating android capture process: {e}")

            
            
            def full_local_capture(self):
                try:
                        
                    self.socket = conf.L2listen(
                        type=ETH_P_ALL
                    )
                    
                    print("[*] doing full local capture")
                    
                    sniff(
                        opened_socket=self.socket,
                        prn=self.write_packet_to_pcap,
                        stop_filter=self.stop_capture_thread
                    )
                except PermissionError as e:
                    print(f"[-] PermissionError: {e}")
                    print("[!] It seems you do not have permissions to access /dev/bpf. Please run the script with 'sudo' or grant your user access to /dev/bpf* files.")
                    print("[!] Exiting the program.")
                    self.clean_up_and_exit()
                except Scapy_Exception as e:
                    print(f"[-] Scapy_Exception: {e}")
                    print("[!] Scapy could not open /dev/bpf for network capture. Ensure you have the correct permissions.")
                    print("[!] Run the script with 'sudo' (not recommended for security reasons).")
                    self.clean_up_and_exit()
                except Exception as e:
                    print(f"[-] Unknown error: {e}")
                    print("[!] Full traceback for debugging:")
                    traceback.print_exc()
                    self.clean_up_and_exit()
                
                
            def run(self):
                if self.is_Mobile:
                    self.mobile_subprocess = self.full_mobile_capture()
                else:
                    self.full_local_capture()
            
            
            def join(self, timeout=None):
                self.stop_capture.set()

                # Terminate the tcpdump process if running
                #if self.android_capture_process and self.android_capture_process.poll() in {None, -2, -15}:
                if self.is_Mobile and pcap_class.android_Instance.is_Android():
                    if self.android_capture_process != -1 and self.android_capture_process.poll() is None:
                        pcap_class.android_Instance.send_ctrlC_over_adb()
                        self.android_capture_process.terminate()
                        try:
                            self.android_capture_process.wait(timeout=2)  # Wait for graceful termination
                        except subprocess.TimeoutExpired:
                            print("[-] Android capture thread did not terminate. Forcing kill.")
                            self.android_capture_process.kill()

                super().join(timeout)
            
            
            def stop_capture_thread(self, packet):
                if hasattr(self.stop_capture, "is_set"):
                    status = self.stop_capture.is_set()
                else:
                    status = self.stop_capture.isSet()
                return status
                
                
            def full_mobile_capture(self):
                if pcap_class.android_Instance.is_Android():
                    if not pcap_class.android_Instance.is_tcpdump_available: 
                        pcap_class.android_Instance.push_tcpdump_to_device()
                    self.android_capture_process = pcap_class.android_Instance.run_tcpdump_capture("_"+self._get_pcap_base_name())
                    
                    print("[*] doing full capture on Android")
                    return self.android_capture_process
                else:
                    print("[-] currently a full capture on iOS is not supported\nAbborting...")
                    exit(2)
                    
        ## End of inner class FullCaptureThread 
        instance_of_thread_class = FullCaptureThread()
        return instance_of_thread_class
   
     
    def write_pcap_header(self, pcap_file):
        self.pcap_file = pcap_file
        for writes in (
            ("=I", 0xa1b2c3d4),     # Magic number
            ("=H", 2),              # Major version number
            ("=H", 4),              # Minor version number
            ("=i", time.timezone),  # GMT to local correction
            ("=I", 0),              # Accuracy of timestamps
            ("=I", 65535),          # Max length of captured packets
            ("=I", 101)):           # Data link type (LINKTYPE_IPV4 = 228) CHANGED TO RAW
            pcap_file.write(struct.pack(writes[0], writes[1]))
        return pcap_file    
    
    def __create_plaintext_pcap(self):
        pcap_file = open(self.pcap_file_name, "wb", 0)
        pcap_file = self.write_pcap_header(pcap_file)
        return pcap_file
    
    def log_plaintext_payload(self, ss_family, function, src_addr, src_port,
                 dst_addr, dst_port, data):
        """Writes the captured data to a pcap file.
        Args:
        pcap_file: The opened pcap file.
        ss_family: The family of the connection, IPv4/IPv6
        function: The function that was intercepted ("SSL_read" or "SSL_write").
        src_addr: The source address of the logged packet.
        src_port: The source port of the logged packet.
        dst_addr: The destination address of the logged packet.
        dst_port: The destination port of the logged packet.
        data: The decrypted packet data.
        """
        
        t = time.time()

        if function in self.SSL_READ:
            session_unique_key = str(src_addr) + str(src_port) + \
                str(dst_addr) + str(dst_port)
        else:
            session_unique_key = str(dst_addr) + str(dst_port) + \
                str(src_addr) + str(src_port)
        if session_unique_key not in self.ssl_sessions:

            self.ssl_sessions[session_unique_key] = (random.randint(0, 0xFFFFFFFF),
                                                random.randint(0, 0xFFFFFFFF))

        client_sent, server_sent = self.ssl_sessions[session_unique_key]

        if function in self.SSL_READ:
            seq, ack = (server_sent, client_sent)
        else:
            seq, ack = (client_sent, server_sent)
        if ss_family == "AF_INET":
            for writes in (
                # PCAP record (packet) header
                # Timestamp seconds
                ("=I", int(t)),
                # Timestamp microseconds
                ("=I", int(t * 1000000) % 1000000),
                # Number of octets saved
                ("=I", 40 + len(data)),
                # Actual length of packet
                ("=i", 40 + len(data)),
                # IPv4 header
                # Version and Header Length
                (">B", 0x45),
                # Type of Service
                (">B", 0),
                # Total Length
                (">H", 40 + len(data)),
                # Identification
                (">H", 0),
                # Flags and Fragment Offset
                (">H", 0x4000),
                # Time to Live
                (">B", 0xFF),
                # Protocol
                (">B", 6),
                # Header Checksum
                (">H", 0),
                (">I", src_addr),                 # Source Address
                (">I", dst_addr),                 # Destination Address
                # TCP header
                (">H", src_port),                 # Source Port
                (">H", dst_port),                 # Destination Port
                (">I", seq),                      # Sequence Number
                (">I", ack),                      # Acknowledgment Number
                (">H", 0x5018),                   # Header Length and Flags
                (">H", 0xFFFF),                   # Window Size
                (">H", 0),                        # Checksum
                    (">H", 0)):                       # Urgent Pointer
                self.pcap_file.write(struct.pack(writes[0], writes[1]))

            self.pcap_file.write(data)

        elif ss_family == "AF_INET6":
            for writes in (
                # PCAP record (packet) header
                # Timestamp seconds
                ("=I", int(t)),
                # Timestamp microseconds
                ("=I", int(t * 1000000) % 1000000),
                # Number of octets saved
                ("=I", 60 + len(data)),
                # Actual length of packet
                ("=i", 60 + len(data)),
                # IPv6 header
                # Version, traffic class and Flow label
                (">I", 0x60000000),
                # Payload length
                (">H", 20 + len(data)),
                # Next Header
                (">B", 6),
                # Hop limit
                (">B", 0xFF),
                # Source Address
                (">16s", bytes.fromhex(src_addr)),
                # Destination Address
                (">16s", bytes.fromhex(dst_addr)),
                # TCP header
                (">H", src_port),                 # Source Port
                (">H", dst_port),                 # Destination Port
                (">I", seq),                      # Sequence Number
                (">I", ack),                      # Acknowledgment Number
                (">H", 0x5018),                   # Header Length and Flags
                (">H", 0xFFFF),                   # Window Size
                (">H", 0),                        # Checksum
                    (">H", 0)):                       # Urgent Pointer
                self.pcap_file.write(struct.pack(writes[0], writes[1]))

            self.pcap_file.write(data)

        else:
            print("Packet has unknown/unsupported family!")

        if function in self.SSL_READ:
            server_sent += len(data)
        else:
            client_sent += len(data)
        self.ssl_sessions[session_unique_key] = (client_sent, server_sent)
        
    
    # creating a filter for scapy or wiresharks display filter depending on the provided socket_trace_set which looks like 
    @staticmethod
    def get_filter_from_traced_sockets(traced_Socket_Set, filter_type="bpf"):
        """
        Generate a filter string from traced sockets.
        
        :param traced_Socket_Set: Set of frozensets containing socket info.
        :param filter_type: "bpf" for BPF filters or "display" for Wireshark display filters.
        :return: Filter string.
        """
        filters = []
        for socket_info in traced_Socket_Set:
            socket_dict = dict(socket_info)  # Convert frozenset back to a dictionary
            src_addr = socket_dict.get("src_addr", "0.0.0.0")
            dst_addr = socket_dict.get("dst_addr", "0.0.0.0")

            if src_addr == "::" or dst_addr == "::" or not src_addr or not dst_addr:
                continue # Skip invalid entries
            
            if filter_type == "bpf":
                filter_part = PCAP.get_bpf_filter(src_addr, dst_addr)
            elif filter_type == "display":
                filter_part = PCAP.get_display_filter(src_addr, dst_addr)
            else:
                raise ValueError("Invalid filter_type. Use 'bpf' or 'display'.")
            
            if filter_part:
                filters.append(filter_part)
        
        return " or ".join(filters)



        
    # this function is able to reduce a capture to the traffic from the traced target application by using the information from the socket trace and applying a bpf filter of those traced packets
    def create_application_traffic_pcap(self, traced_Socket_Set,pcap_obj,  is_verbose=False):
        def is_valid_socket(socket_info):
            return (
                socket_info.get("src_addr") and socket_info.get("dst_addr")
                and socket_info.get("src_addr") != INVALID_IPV4
                and socket_info.get("dst_addr") != INVALID_IPV4
                and socket_info.get("src_addr") != INVALID_IPV6
                and socket_info.get("dst_addr") != INVALID_IPV6
            )

        if not traced_Socket_Set:
            print("[-] No sockets traced. The resulting PCAP will contain all traffic from the device.")
            return

        # Convert each frozenset in the traced_Socket_Set back to a dictionary
        socket_dicts = [dict(frozenset_entry) for frozenset_entry in traced_Socket_Set]

        valid_sockets = [socket for socket in socket_dicts if is_valid_socket(socket)]
        if not valid_sockets:
            print("[-] No valid sockets found. The resulting PCAP will contain all traffic.")
            return

        bpf_filter = PCAP.get_filter_from_traced_sockets(valid_sockets, filter_type="bpf")
        if not bpf_filter:
            print("[-] Failed to generate a valid BPF filter.")
            return

        if is_verbose:
            print(f"[*] Filtering with BPF filter:\n{bpf_filter}")
        try:
            """
            There is currently a bug which is happening when invoking sniff. Currently we just ignore this warning:
            Exception ignored in: <function Popen.__del__ at 0x10ad64180>
            Traceback (most recent call last):
            File ".../subprocess.py", line 1127, in __del__
                _warn("subprocess %s is still running" % self.pid,
            ResourceWarning: subprocess 63901 is still running
            reading from file <name>.pcap, link-type LINUX_SLL2 (Linux cooked v2)
            """
            filtered_capture = sniff(offline="_" + self.pcap_file_name, filter=bpf_filter) 
            wrpcap(self.pcap_file_name, filtered_capture)
        except Exception as e:
            print(f"[-] Error during PCAP filtering: {e}")
        else:
            print(f"[*] Successfully filtered. Output written to {self.pcap_file_name}")

    
    
    def get_pcap_name(self):
        return self.pcap_file_name
    
    
    @staticmethod
    def get_display_filter(src_addr,dst_addr):
        return f"ip.src == {src_addr} and ip.dst == {dst_addr}"
    
    
    @staticmethod
    def get_bpf_filter(src_addr,dst_addr):
        if src_addr == "::" or dst_addr == "::" or not src_addr or not dst_addr:
            return ""  # Skip invalid entries
        return f"(src host {src_addr} and dst host {dst_addr})"






#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import frida
import tempfile
import os
import struct
import socket
import pprint
import signal
import time
import sys
import json
import threading
from .pcap import PCAP
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler

try:
    import hexdump  # pylint: disable=g-import-not-at-top
except ImportError:
    print("Unable to import hexdump module!")
    pass

try:
    from rich.console import Console
    from rich.panel import Panel
    from rich.text import Text
except ImportError:
    Console = None


# here - where we are.
here = os.path.abspath(os.path.dirname(__file__))

# Names of all supported read functions:
SSL_READ = ["SSL_read", "wolfSSL_read", "readApplicationData", "NSS_read","Full_read"]
# Names of all supported write functions:
SSL_WRITE = ["SSL_write", "wolfSSL_write", "writeApplicationData", "NSS_write","Full_write"]

class SSL_Logger():

    def __init__(self, app, pcap_name=None, verbose=False, spawn=False, keylog=False, enable_spawn_gating=False, mobile=False, live=False, environment_file=None, debug_mode=False,full_capture=False, socket_trace=False, host=False, offsets=None, debug_output=False, experimental=False, anti_root=False, payload_modification=False,enable_default_fd=False, patterns=None, custom_hook_script=None):
        self.debug = debug_mode
        self.anti_root = anti_root
        self.pcap_name = pcap_name
        self.mobile = mobile
        self.debug_output = debug_output
        self.full_capture = full_capture
        self.target_app = app
        self.verbose = verbose
        self.spawn = spawn
        self.pcap_obj = None
        self.socket_trace = socket_trace
        self.keylog = keylog
        self.offsets = offsets
        self.offsets_data = None
        self.environment_file = environment_file
        self.host = host
        self.enable_spawn_gating = enable_spawn_gating
        self.live = live
        self.payload_modification = payload_modification
        self.enable_default_fd = enable_default_fd
        self.experimental = experimental
        self.custom_hook_script = custom_hook_script
        self.script = None
        self.running = True

        self.tmpdir = None
        self.filename = ""
        self.startup = True

        self.process = None
        self.device = None
        self.keylog_file = None

        self.patterns = patterns
        self.pattern_data = None        

        self.keydump_Set = {*()}
        self.traced_Socket_Set = {*()}
        self.traced_scapy_socket_Set = {*()}

        self.init_fritap()
    
    
    def init_fritap(self):
        if frida.__version__ < "17":
            self.frida_agent_script = "_ssl_log_legacy.js"
        else:
            self.frida_agent_script = "_ssl_log.js"

        if self.pcap_name:
            self.pcap_obj =  PCAP(self.pcap_name,SSL_READ,SSL_WRITE,self.full_capture, self.mobile,self.debug)

        if self.offsets is not None:
            if os.path.exists(self.offsets):
                offset_file = open(self.offsets, "r")
                self.offsets_data = offset_file.read()
                offset_file.close()
            else:
                try:
                    json.load(self.offsets)
                    self.offsets_data = self.offsets
                except ValueError as e:
                    print(f"Log error, defaulting to auto-detection: {e}")

        if self.patterns is not None:
            self.load_patterns()

        if self.keylog:
            self.keylog_file = open(self.keylog, "w")

        if self.live:
            if self.pcap_name:
                print("[*] YOU ARE TRYING TO WRITE A PCAP AND HAVING A LIVE VIEW\nTHIS IS NOT SUPPORTED!\nWHEN YOU DO A LIVE VIEW YOU CAN SAFE YOUR CAPUTRE WITH WIRESHARK.")
            fifo_file = self.temp_fifo()
            print('[*] friTap live view on Wireshark')
            print(f'[*] Created named pipe for Wireshark live view to {fifo_file}')
            print(
                f'[*] Now open this named pipe with Wireshark in another terminal: sudo wireshark -k -i {fifo_file}')
            print('[*] friTap will continue after the named pipe is ready....\n')
            self.pcap_obj =  PCAP(fifo_file,SSL_READ,SSL_WRITE,self.full_capture, self.mobile,self.debug)

    
    def on_detach(self, reason):

        if reason == "application-requested":
            return
        
        print(f"\n[*] Target process stopped: {reason}\n")            
        self.pcap_cleanup(self.full_capture,self.mobile,self.pcap_name)
        self.cleanup(self.live,self.socket_trace,self.full_capture,self.debug)
        

    def handle_frida_script_error(self, message: dict):
        print("\n\n")
        error_msg = message.get("description", "Unknown error")
        stack = message.get("stack", "No stacktrace provided")
        file = message.get("fileName", "")
        line = message.get("lineNumber", "")
        column = message.get("columnNumber", "")

        if self.debug_output:
            if Console:
                console = Console()
                header = Text("✖ Frida Script Error", style="bold red")
                body = Text.from_markup(
                    f"[bold]Description:[/bold] {error_msg}\n"
                    f"[bold]File:[/bold] {file}:{line}:{column}\n\n"
                    f"[bold]Stacktrace:[/bold]\n{stack}"
                )
                panel = Panel(body, title=header, expand=False, border_style="red")
                console.print(panel)
            else:
                print("✖ Frida Script Error:")
                print("Description:", error_msg)
                print(f"File: {file}:{line}:{column}")
                print("Stacktrace:\n" + stack)
        else:
            print(f"[!] Error from Frida script: {error_msg}")
        
        print("\n\n[!] Exiting due to script error.")

        os.kill(os.getpid(), signal.SIGTERM)

    def temp_fifo(self):
        self.tmpdir = tempfile.mkdtemp()
        self.filename = os.path.join(self.tmpdir, 'fritap_sharkfin')  # Temporary filename
        os.mkfifo(self.filename)  # Create FIFO
        try:
            return self.filename
        except OSError as e:
            print(f'Failed to create FIFO: {e}')
        

    def on_fritap_message(self, job, message, data):
        """Callback for errors and messages sent from Frida-injected JavaScript.
        Logs captured packet data received from JavaScript to the console and/or a
        pcap file. See https://www.frida.re/docs/messages/ for more detail on
        Frida's messages.
        Args:
        message: A dictionary containing the message "type" and other fields
            dependent on message type.
        data: The string of captured decrypted data or the caputured decryption keys
        """


        """
        This offers the possibility to work with the JobManger() from the AndroidFridaManager project.
        """
        if self.script is None:
            self.script = job.script

        #print("----- Debug message -----")
        #print(message)
        #print("-------------------------")
        msg_type = message.get('type')
        #print(f"[*] Received message of type: {msg_type}")

        if msg_type == 'send':
            payload = message.get('payload')

            if self.startup and payload == 'experimental':
                self.script.post({'type':'experimental', 'payload': self.experimental})

            if self.startup and payload == 'defaultFD':
                self.script.post({'type':'defaultFD', 'payload': self.enable_default_fd})

            if self.startup and payload == 'socket_tracing':
                self.script.post({'type':'socket_tracing', 'payload': self.socket_trace})

            if self.startup and payload == 'pattern_hooking':
                self.script.post({'type':'pattern_hooking', 'payload': self.pattern_data})

            if self.startup and payload == 'offset_hooking':
                self.script.post({'type':'offset_hooking', 'payload': self.offsets_data})
            
            if self.startup and payload == 'anti':
                self.script.post({'type':'antiroot', 'payload': self.anti_root})
                self.startup = False
            
        
        if message["type"] == "error":
            self.handle_frida_script_error(message)
            return
        
        if "contentType" not in payload:
            return
        if payload["contentType"] == "console":
            if payload["console"].startswith("[*]"):
                print(payload["console"])
            else:
                print("[*] " + payload["console"])
        if self.debug or self.debug_output:
            if payload["contentType"] == "console_dev" and payload["console_dev"]:
                if len(payload["console_dev"]) > 3:
                    print("[***] " + payload["console_dev"])
            elif payload["contentType"] == "console_error" and payload["console_error"]:
                if len(payload["console_error"]) > 3:
                    print("[---] " + payload["console_error"])
        if self.verbose:
            if(payload["contentType"] == "keylog") and self.keylog:
                if payload["keylog"] not in self.keydump_Set:
                    print(payload["keylog"])
                    self.keydump_Set.add(payload["keylog"])
                    self.keylog_file.write(payload["keylog"] + "\n")
                    self.keylog_file.flush()    
            elif not data or len(data) == 0:
                return
            else:
                src_addr = get_addr_string(payload["src_addr"], payload["ss_family"])
                dst_addr = get_addr_string(payload["dst_addr"], payload["ss_family"])

                if not self.socket_trace and not self.full_capture:
                    print("SSL Session: " + str(payload["ssl_session_id"]))

                if self.full_capture:
                    # Add to traced_scapy_socket_Set as a frozenset dictionary
                    scapy_filter_entry = {
                        "src_addr": src_addr,
                        "dst_addr": dst_addr,
                        "ss_family": payload["ss_family"]
                    }
                    self.traced_scapy_socket_Set.add(frozenset(scapy_filter_entry.items()))  # Use frozenset for uniqueness

                if self.socket_trace:
                    display_filter_entry = {
                        "src_addr": src_addr,
                        "dst_addr": dst_addr,
                        "src_port": payload["src_port"],
                        "dst_port": payload["dst_port"],
                        "ss_family": payload["ss_family"]
                    }
                    self.traced_Socket_Set.add(frozenset(display_filter_entry.items()))
                    scapy_filter_entry = {
                        "src_addr": src_addr,
                        "dst_addr": dst_addr,
                        "ss_family": payload["ss_family"]
                    }
                    self.traced_scapy_socket_Set.add(frozenset(scapy_filter_entry.items()))

                    # Use structured data for the debug print
                    print(f"[socket_trace] {src_addr}:{payload['src_port']} --> {dst_addr}:{payload['dst_port']}")

                else:
                    print("[%s] %s:%d --> %s:%d" % (payload["function"], src_addr, payload["src_port"], dst_addr, payload["dst_port"]))
                    hexdump.hexdump(data)
                print()
        if self.pcap_name and payload["contentType"] == "datalog" and not self.full_capture:
            self.pcap_obj.log_plaintext_payload(payload["ss_family"], payload["function"], payload["src_addr"],
                     payload["src_port"], payload["dst_addr"], payload["dst_port"], data)
        if self.live and payload["contentType"] == "datalog" and not self.full_capture:
            try:
                self.pcap_obj.log_plaintext_payload(payload["ss_family"], payload["function"], payload["src_addr"],
                         payload["src_port"], payload["dst_addr"], payload["dst_port"], data)
            except (BrokenPipeError, IOError):
                self.detach_with_timeout(self.process)
                self.cleanup(self.live, self.socket_trace, self.full_capture, self.debug)

        if self.keylog and payload["contentType"] == "keylog":
            if payload["keylog"] not in self.keydump_Set:
                self.keylog_file.write(payload["keylog"] + "\n")
                self.keylog_file.flush()
                self.keydump_Set.add(payload["keylog"])
        
        if self.socket_trace or self.full_capture:
            if "src_addr" not in payload:
                return
            
            src_addr = get_addr_string(payload["src_addr"], payload["ss_family"])
            dst_addr = get_addr_string(payload["dst_addr"], payload["ss_family"])

            if self.socket_trace:
                # Add a structured dictionary to traced_Socket_Set
                display_filter_entry = {
                    "src_addr": src_addr,
                    "dst_addr": dst_addr,
                    "src_port": payload["src_port"],
                    "dst_port": payload["dst_port"],
                    "ss_family": payload["ss_family"]
                }
                self.traced_Socket_Set.add(frozenset(display_filter_entry.items()))  # Use frozenset for uniqueness
                # Add a structured dictionary to traced_scapy_socket_Set
                scapy_filter_entry = {
                    "src_addr": src_addr,
                    "dst_addr": dst_addr,
                    "ss_family": payload["ss_family"]
                }
                self.traced_scapy_socket_Set.add(frozenset(scapy_filter_entry.items()))  # Use frozenset for uniqueness

    

    
    def on_custom_hook_message(self, message, data):
        """
        This handler is used to print the messages from the provided custom hooks
        """

        if message["type"] == "error":
            pprint.pprint(message)
            os.kill(os.getpid(), signal.SIGTERM)
            return
        
        custom_hook_payload = message["payload"]
        if "custom" not in custom_hook_payload:
            return

        print("[+] custom hook: "+custom_hook_payload["custom"])

    
    
    def on_child_added(self, child):
        print(f"[*] Attached to child process with pid {child.pid}")
        self.instrument(self.device.attach(child.pid))
        self.device.resume(child.pid)


    def on_spawn_added(self, spawn):
        print(
            f"[*] Process spawned with pid {spawn.pid}. Name: {spawn.identifier}")
        self.instrument(self.device.attach(spawn.pid))
        self.device.resume(spawn.pid)
        

    def instrument(self, process, own_message_handler):
        runtime="qjs"
        debug_port = 1337
        if self.debug:
            if frida.__version__ < "16":
                process.enable_debugger(debug_port)
            print("\n[!] running in debug mode")
            print(f"[!] Chrome Inspector server listening on port {debug_port}")
            print("[!] Open Chrome with chrome://inspect for debugging\n")
            runtime="v8"
        
        if self.custom_hook_script is not None:
            custom_script_string = self.get_custom_frida_script()
            if self.debug_output:
                print("[***] loading custom frida script: " + self.custom_hook_script)
        
        script_string = self.get_fritap_frida_script()
        if self.debug_output:
            print("[***] loading friTap frida script: " + self.frida_agent_script)
        

        if self.offsets_data is not None:
            print(f"[*] applying hooks at offset {self.offsets_data}")


        if self.pattern_data is not None:
            print("[*] Using pattern provided by pattern.json for hooking")

        
        if self.custom_hook_script is not None:
            self.custom_script = process.create_script(custom_script_string, runtime=runtime)
            self.custom_script.on("message", self.on_custom_hook_message)
            self.custom_script.load()


        self.script = process.create_script(script_string, runtime=runtime)

        if self.debug and frida.__version__ >= "16":
            self.script.enable_debugger(debug_port)

        if own_message_handler is not None:
            self.script.on("message", self._provide_custom_hooking_handler(own_message_handler))
            return self.script
        else:
            self.script.on("message", self._internal_callback_wrapper())
        self.script.load()
        
        

        
        #script.post({'type':'readmod', 'payload': '0x440x410x53'})
        if self.payload_modification:
            class ModWatcher(FileSystemEventHandler):
                def __init__(self, process):
                    
                    self.process = process

                def on_any_event(self, event):
                    try:
                        if(event.event_type == "modified" and ("readmod" in event.src_path)):
                            with open("./readmod.bin", "rb") as f:
                                buffer = f.read()
                                self.script.post({'type':'readmod', 'payload': buffer.hex()})
                        elif(event.event_type == "modified" and ("writemod" in event.src_path)):
                            with open("./writemod.bin", "rb") as f:
                                buffer = f.read()
                                self.script.post({'type':'writemod', 'payload': buffer.hex()})
                    except RuntimeError as e:
                        print(e)
                
                

            print("Init watcher")
            event_handler = ModWatcher(process)
            
            observer = Observer()
            observer.schedule(event_handler, os.getcwd())
            observer.start()

        return self.script
     
    
    def load_patterns(self):
        if os.path.exists(self.patterns):
            try:
                with open(self.patterns, "rb") as file:
                    raw_data = file.read()
                    pattern_file = raw_data.decode("utf-8")

                # Try parsing the JSON to verify it's valid
                json_data = json.loads(pattern_file)
                self.pattern_data = json.dumps(json_data, ensure_ascii=False).encode('utf-8').decode('utf-8')  # Ensure pattern_data is a JSON string in utf-8

            except (UnicodeDecodeError, json.JSONDecodeError) as e:
                print(f"[-] UnicodeDecodeError: {e}")
            except (ValueError, json.JSONDecodeError) as e:
                print(f"[-] Error loading JSON file, defaulting to auto-detection: {e}")
            except OSError as e:
                print(f"[-] Error reading the file: {e}")
        else:
            print(f"[-] Pattern file {self.patterns} does not exist.")
    
    
    def start_fritap_session_instrumentation(self, own_message_handler, process):
        self.process = process
        script = self.instrument(self.process, own_message_handler)
        return script

    
    def start_fritap_session(self, own_message_handler=None):

        if self.mobile:
            try:
                if self.mobile is True:  # No device ID provided
                    if self.debug_output or self.debug:
                        print("[*] Attaching to the first available USB device...")
                    self.device = frida.get_usb_device()
                else:  # Device ID provided
                    if self.debug_output or self.debug:
                        print(f"[*] Attaching to the device with ID: {self.mobile}")
                    self.device = frida.get_device(self.mobile)
                print("[*] Successfully attached to the mobile device.")
            except frida.ServerNotRunningError:
                print("[-] Frida server is not running. Please ensure it is started on the device.")
                sys.exit(1)
            except frida.DeviceNotFoundError:
                print(f"[-] Device with ID '{self.mobile}' not found. Please check the device ID or ensure it is connected.")
                sys.exit(1)
            except Exception as e:
                print(f"[-] Unexpected error while attaching to the device: {e}")
                sys.exit(1)
        elif self.host:
            self.device = frida.get_device_manager().add_remote_device(self.host)
        else:
            self.device = frida.get_local_device()

        self.device.on("child_added", self.on_child_added)
        if self.enable_spawn_gating:
            self.device.enable_spawn_gating()
            self.device.on("spawn_added", self.on_spawn_added)
        if self.spawn:
            print("spawning "+ self.target_app)
            
            if self.mobile or self.host:
                pid = self.device.spawn(self.target_app)
            else:
                used_env = {}
                if self.environment_file:
                    with open(self.environment_file) as json_env_file:
                        used_env = json.load(json_env_file)
                pid = self.device.spawn(self.target_app.split(" "),env=used_env)
                self.device.resume(pid)
                time.sleep(1) # without it Java.perform silently fails
            self.process = self.device.attach(pid)
        else:
            self.process = self.device.attach(int(self.target_app) if self.target_app.isnumeric() else self.target_app)


        script = self.instrument(self.process, own_message_handler)



        if self.pcap_name and self.full_capture:
            print(f'[*] Logging pcap to {self.pcap_name}')
        if self.pcap_name and not self.full_capture:
            print(f'[*] Logging TLS plaintext as pcap to {self.pcap_name}')
        if self.keylog:
            print(f'[*] Logging keylog file to {self.keylog}')
            
        #self.process.on('detached', self.on_detach)

        if self.spawn:
            self.device.resume(pid)

        return self.process, script


    def finish_fritap(self):
        if self.script:
            self.script.unload()


    def _provide_custom_hooking_handler(self, handler):
        def wrapped_handler(message, data):
            handler(message, data)

        return wrapped_handler


    def _internal_callback_wrapper(self):
        def wrapped_handler(message, data):
            self.on_fritap_message(None, message, data)
        
        return wrapped_handler


    def detach_with_timeout(self, timeout=5):
        """
        Attempt to detach from the Frida process with a timeout.

        Args:
            process: The Frida process to detach from.
            timeout: Time in seconds to wait before forcing detachment.
        """
        def detach():
            try:
                if self.debug_output or self.debug:
                    print("[*] Attempting to detach from Frida process...")
                try:
                    self.script.unload()
                except Exception:
                    pass

                self.process.detach()
                if self.debug_output or self.debug:
                    print("[*] Successfully detached from Frida process.")
            except Exception as e:
                print(f"[-] Error while detaching: {e}")

        # Create a thread to run the detach method
        detach_thread = threading.Thread(target=detach)
        detach_thread.start()

        # Wait for the thread to complete
        detach_thread.join(timeout=timeout)

        if detach_thread.is_alive():
            if self.debug_output:
                print(f"[-] Detach process timed out after {timeout} seconds.")
            # Force cleanup if necessary
            # Note: Frida doesn't provide a "force detach," so handle gracefully
        else:
            if self.debug_output:
                print("[*] Detached friTap from process successfully.")



    def set_keylog_file(self, keylog_name):
        self.keylog_file = open(keylog_name, "w")
    

    def pcap_cleanup(self, is_full_capture, is_mobile, pcap_name):
        if is_full_capture and self.pcap_obj is not None:
                capture_type = "local"
                self.pcap_obj.full_capture_thread.join(2.0)
                if self.pcap_obj.full_capture_thread.is_alive() and not is_mobile:
                    self.pcap_obj.full_capture_thread.socket.close()
                if self.pcap_obj.full_capture_thread.mobile_subprocess != -1:
                    capture_type = "mobile"
                    self.pcap_obj.android_Instance.send_ctrlC_over_adb()
                    time.sleep(1)
                    self.pcap_obj.full_capture_thread.mobile_subprocess.terminate()
                    self.pcap_obj.full_capture_thread.mobile_subprocess.wait()
                    if not self.pcap_obj.android_Instance.is_tcpdump_available():
                        print("[-] tcpdump is not available on the device.")
                        return
                    self.pcap_obj.android_Instance.pull_pcap_from_device()
                print(f"[*] full {capture_type} capture safed to _{pcap_name}")
                if self.keylog_file is None:
                    print("[*] remember that the full capture won't contain any decrypted TLS traffic.")
                else:
                    print(f"[*] remember that the full capture won't contain any decrypted TLS traffic. In order to decrypt it use the logged keys from {self.keylog_file.name}")
    

    def cleanup(self, live=False, socket_trace=False, full_capture=False, debug_output=False, debug=False):
        if self.pcap_obj is not None and full_capture:
            if self.pcap_obj.full_capture_thread.is_alive():
                self.pcap_obj.full_capture_thread.join()
                time.sleep(2)
        if live:
            os.unlink(self.filename)  # Remove file
            os.rmdir(self.tmpdir)  # Remove directory
        if type(socket_trace) is str:
            print(f"[*] Write traced sockets into {socket_trace}")
            self.write_socket_trace(socket_trace)
        if socket_trace:
            display_filter = PCAP.get_filter_from_traced_sockets(self.traced_Socket_Set, filter_type="display")
            print(f"[*] Generated Display Filter for Wireshark:\n{display_filter}")
        
        if full_capture and len(self.traced_scapy_socket_Set) > 0:
            if debug_output or debug:
                display_filter = PCAP.get_filter_from_traced_sockets(self.traced_Socket_Set, filter_type="display")
                print(f"[*] Generated Display Filter for Wireshark:\n{display_filter}")

            try:
                self.pcap_obj.create_application_traffic_pcap(self.traced_scapy_socket_Set,self.pcap_obj)
            except Exception as e:
                print(f"Error: {e}")

        elif full_capture and len(self.traced_scapy_socket_Set) < 1:
            if socket_trace:
                print(f"[-] friTap was unable to indentify the used sockets. \n[*] The resulting PCAP _{self.pcap_obj.pcap_file_name} will contain all trafic from the device.")
            else:
                print(f"[*] friTap not trace the sockets in use (--socket_tracing option not enabled)\n[*] The resulting PCAP _{self.pcap_obj.pcap_file_name} will contain all trafic from the device.")
            
        self.running = False
        if self.process:
            self.detach_with_timeout()  # Detach Frida process if applicable
        print("\n\nThx for using friTap\nHave a great day\n")
        os._exit(0)

    def get_fritap_frida_script(self):
        with open(os.path.join(here, self.frida_agent_script), encoding='utf-8', newline='\n') as f:
            script_string = f.read()
            return script_string
            
    
    def get_custom_frida_script(self):
        with open(os.path.join(here, self.custom_hook_script), encoding='utf-8', newline='\n') as f:
            script_string = f.read()
            return script_string
    
    
    def get_fritap_frida_script_path(self):
        return os.path.join(os.path.dirname(__file__), self.frida_agent_script)


    def install_signal_handler(self):
        def signal_handler(signum, frame):
            print("\n[*] Ctrl+C detected. Cleaning up...")
            self.pcap_cleanup(self.full_capture, self.mobile, self.pcap_name)
            self.cleanup(self.live, self.socket_trace, self.full_capture, self.debug_output, self.debug)  # Call the instance's cleanup method


        signal.signal(signal.SIGINT, signal_handler)


    def write_socket_trace(self, socket_trace_name):
        with open(socket_trace_name, 'a') as trace_file:
            trace_file.write(PCAP.get_filter_from_traced_sockets(self.traced_Socket_Set, filter_type="display") + '\n')


  
def get_addr_string(socket_addr,ss_family):
    if ss_family == "AF_INET":
        return  socket.inet_ntop(socket.AF_INET, struct.pack(">I", socket_addr))
    else: # this should only be AF_INET6
        raw_addr = bytes.fromhex(socket_addr)
        return socket.inet_ntop(socket.AF_INET6, struct.pack(">16s", raw_addr))






#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import sys
import os
import frida
from AndroidFridaManager import FridaBasedException
import traceback
from .about import __version__
from .about import __author__
from .ssl_logger import SSL_Logger



# usually not needed - but sometimes the replacements of the script result into minor issues
# than we have to look into the generated final frida script we supply
def write_debug_frida_file(debug_script_version):
    debug_script_file = "_ssl_log_debug.js"
    f = open(debug_script_file, 'wt', encoding='utf-8')
    f.write(debug_script_version)
    f.close()
    print(f"[!] written debug version of the frida script: {debug_script_file}")



class ArgParser(argparse.ArgumentParser):
    def error(self, message):
        print("friTap v" + __version__)
        print("by " + __author__)
        print()
        print("Error: " + message)
        print()
        print(self.format_help().replace("usage:", "Usage:"))
        self.exit(0)


def main():

    parser = ArgParser(
        add_help=False,
        description="Decrypts and logs an executables or mobile applications SSL/TLS traffic.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        allow_abbrev=False,
        epilog=r"""
Examples:
  %(prog)s -m -p ssl.pcap com.example.app
  %(prog)s -m --pcap log.pcap --verbose com.example.app
  %(prog)s -m -k keys.log -v -s com.example.app
  %(prog)s -m -k keys.log -v -c <path to custom hook script> -s com.example.app
  %(prog)s -m --patterns pattern.json -k keys.log -s com.google.android.youtube
  %(prog)s --pcap log.pcap "$(which curl) https://www.google.com"
  %(prog)s -H --pcap log.pcap 192.168.0.1:1234 com.example.app
  %(prog)s -m -p log.pcap --enable_spawn_gating -v -do -sot --full_capture -k keys.log com.example.app
  %(prog)s -m -p log.pcap --enable_spawn_gating -v -do --anti_root --full_capture -k keys.log com.example.app
  %(prog)s -m -p log.pcap --enable_default_fd com.example.app
""")

    args = parser.add_argument_group("Arguments")
    args.add_argument("-m", "--mobile",  metavar="<device_id>", default=False, required=False, nargs='?', const=True, help="Attach to a process on Android or iOS. If you have multpile device you specify with the device id.")
    args.add_argument("-H", "--host", metavar="<ip:port>", required=False,
                      help="Attach to a process on remote frida device")
    args.add_argument("-c", "--custom_script", metavar="<path>", required=False,
                      help="Path to the custom Frida script that will be executed prior to applying the friTap hooks.")
    args.add_argument("-d", "--debug", required=False, action="store_const", const=True,
                      help="Set friTap into debug mode this include debug output as well as a listening Chrome Inspector server for remote debugging.")
    args.add_argument("-do", "--debugoutput", required=False, action="store_const", const=True,
                      help="Activate the debug output only.")
    args.add_argument("-ar", "--anti_root", required=False, action="store_const", const=True, default=False, help="Activate anti root hooks for Android")
    args.add_argument("-ed", "--enable_default_fd", required=False, action="store_const", const=True, default=False, help="Activate the fallback socket information (127.0.0.1:1234-127.0.0.1:2345) whenever the file descriptor (FD) of the socket cannot be determined")
    args.add_argument("-f", "--full_capture", required=False, action="store_const", const=True, default=False,
                      help="Do a full packet capture instead of logging only the decrypted TLS payload. Set pcap name with -p <PCAP name>")
    args.add_argument("-k", "--keylog", metavar="<path>", required=False,
                      help="Log the keys used for tls traffic")
    args.add_argument("-l", "--live", required=False, action="store_const", const=True,
                      help="Creates a named pipe /tmp/sharkfin which can be read by Wireshark during the capturing process")
    args.add_argument("-p", "--pcap", metavar="<path>", required=False,
                      help="Name of PCAP file to write")
    args.add_argument("-s", "--spawn", required=False, action="store_const", const=True,
                      help="Spawn the executable/app instead of attaching to a running process")
    args.add_argument("-sot", "--socket_tracing", metavar="<path>", required=False, nargs='?', const=True,
                      help="Traces all socket of the target application and provide a prepared wireshark display filter. If pathname is set, it will write the socket trace into a file-")
    args.add_argument("-env","--environment", metavar="<env.json>", required=False,
                      help="Provide the environment necessary for spawning as an JSON file. For instance: {\"ENV_VAR_NAME\": \"ENV_VAR_VALUE\" }")
    args.add_argument("-v", "--verbose", required=False, action="store_const",
                      const=True, help="Show verbose output")
    args.add_argument('--version', action='version',version='friTap v{version}'.format(version=__version__))
    args.add_argument("--enable_spawn_gating", required=False, action="store_const", const=True,
                      help="Catch newly spawned processes. ATTENTION: These could be unrelated to the current process!")
    args.add_argument("exec", metavar="<executable/app name/pid>",
                      help="executable/app whose SSL calls to log")
    args.add_argument("--offsets", required=False, metavar="<offsets.json>",
                      help="Provide custom offsets for all hooked functions inside a JSON file or a json string containing all offsets. For more details see our example json (offsets_example.json)")
    args.add_argument("--patterns", required=False, metavar="<pattern.json>",
                  help="Provide custom patterns for module hooking inside a JSON file or a JSON string containing platform-specific patterns. For more details see our provided JSON (pattern.json)")
    args.add_argument("--payload_modification", required=False, action="store_const", const=True, default=False,
                      help="Capability to alter the decrypted payload. Be careful here, because this can crash the application.")                  
    args.add_argument("-exp","--experimental", required=False, action="store_const", const=True, default=False,
                      help="Activates all existing experimental feature (see documentation for more information)")
    parsed = parser.parse_args()

    
    if parsed.full_capture and parsed.pcap is None:
        parser.error("--full_capture requires -p to set the pcap name")
        exit(2)

    if parsed.full_capture and parsed.keylog is None:
        print("[*] Are you sure you want to proceed without recording the key material (-k <keys.log>)?\n[*] Without the key material, you have a complete network record, but no way to view the contents of the TLS traffic.")
        print("[*] Do you want to proceed without recording keys? : <press any key to proceed or Strg+C to abort>")
        input() 
    try:
        print("Start logging")
        print("Press Ctrl+C to stop logging\n")
        ssl_log = SSL_Logger(parsed.exec, parsed.pcap, parsed.verbose,
                parsed.spawn, parsed.keylog, parsed.enable_spawn_gating, parsed.mobile, parsed.live, parsed.environment, parsed.debug, parsed.full_capture, parsed.socket_tracing, parsed.host, parsed.offsets, parsed.debugoutput, parsed.experimental, parsed.anti_root, parsed.payload_modification, parsed.enable_default_fd, parsed.patterns, parsed.custom_script)

        ssl_log.install_signal_handler()        
        ssl_log.start_fritap_session()  
        
        # Wait for user input or interrupt
        while ssl_log.running:
            pass
    except frida.TransportError as fe:
        print(f"[-] Problems while attaching to frida-server: {fe}")
        exit(2)
    except FridaBasedException as e:
        print(f"[-] Frida based error: {e}")
        exit(2)
    except frida.TimedOutError as te:
        print(f"[-] TimeOutError: {te}")
        exit(2)
    except frida.ProcessNotFoundError as pe:
        print(f"[-] ProcessNotFoundError: {pe}")
        exit(2)
    except frida.PermissionDeniedError as e:
        print(f"[-] Frida Permission Denied: {e}")
        exit(2)
    except frida.ServerNotRunningError as e:
        print(f"[-] Frida server is not running: {e}")
        exit(2)
    except frida.InvalidArgumentError as e:
        print(f"[-] Invalid Argument passed to Frida: {e}")
        if "device not found" in e:
            print("[-] Frida is unable to identify the target device.")
            print("[-] If you have multiple devices connected, please specify the device ID using the `-m` option:")
            print("\t1. Identify the target device ID (e.g., using `adb devices` or `frida-ls-devices`).")
            print("\t2. Run FriTap with the device ID:")
            print("\t   fritap -m <device-id> <target>")

        exit(2)
    except frida.InvalidOperationError as e:
        print(f"[-] Invalid Operation Error in Frida: {e}")
        exit(2)
    except Exception as ar:
        # Get current system exception
        ex_type, ex_value, ex_traceback = sys.exc_info()
        
        # Extract unformatter stack traces as tuples
        trace_back = traceback.extract_tb(ex_traceback)
        
        # Format stacktrace
        stack_trace = list()
        
        for trace in trace_back:
            stack_trace.append("File : %s , Line : %d, Func.Name : %s, Message : %s" % (trace[0], trace[1], trace[2], trace[3]))
        
        if parsed.debug or parsed.debugoutput:
            if "NotSupportedError" in ex_type.__name__:
                print("[-] Frida based error:")
            print("[-] Exception type : %s " % ex_type.__name__)
            print("[-] Exception message : %s" %ex_value)
            print("[-] Stack trace : %s" %stack_trace)


        if "unable to connect to remote frida-server: closed" in str(ar):
            print("\n[-] frida-server is not running in remote device. Please run frida-server and rerun")
            
        if "NotSupportedError" in ex_type.__name__:
            print(f"\n[-] Frida error: {ex_value}")
        else:
            print(f"\n[-] Unknown error: {ex_value}")

        if "unable to access process with pid" in str(ex_value).lower():
            print("\n\nThx for using friTap\nHave a great day\n")
            os._exit(0)
        if "not yet supported on this os" in str(ex_value).lower():
            print("[-] This feature is currently not supported by frida on this OS.")
            print("\n\nThx for using friTap\nHave a great day\n")
            os._exit(0)

        ssl_log.pcap_cleanup(parsed.full_capture,parsed.mobile,parsed.pcap)
        ssl_log.cleanup(parsed.live,parsed.socket_tracing,parsed.full_capture,parsed.debug,parsed.debugoutput)    

    finally:
        if isinstance(ssl_log, SSL_Logger):
            ssl_log.pcap_cleanup(parsed.full_capture,parsed.mobile,parsed.pcap)
            ssl_log.cleanup(parsed.live,parsed.socket_tracing,parsed.full_capture,parsed.debug)
        

if __name__ == "__main__":
    main()


import os
from setuptools import setup, find_packages
from os.path import abspath, dirname, join

# Fetches the content from README.md
# This will be used for the "long_description" field.
README_MD = open(join(dirname(abspath(__file__)), "README.md")).read()

# here - where we are.
here = os.path.abspath(os.path.dirname(__file__))

# Path to the about.py file
init_py_path = join(here, "friTap", "about.py")

# Read version and author from about.py
with open(init_py_path) as f:
    exec(f.read())

# read the package requirements for install_requires
#with open(os.path.join(here, 'requirements.txt'), 'r') as f:
#    requirements = f.readlines()

def get_version():
    about = {}
    with open(init_py_path) as f:
        exec(f.read(), about)
    return about["__version__"]

def get_author():
    author = {}
    with open(init_py_path) as f:
        exec(f.read(), author)
    return author["__author__"]


# Define dependencies directly in setup.py
requirements = [
    'frida>=16.0.0',
    'frida-tools>=11.0.0',
    'AndroidFridaManager',
    'hexdump',
    'scapy',
    'watchdog',
    'click',
    'importlib-resources',
    'psutil',
    'rich>=13.0.0',
]


setup(
    # pip install friTap
    name="friTap",
    version=get_version(),  # Dynamically get the version from about.py

    # The description that will be shown on PyPI.
    description="Simplifying SSL/TLS traffic analysis for researchers by making SSL/TLS decryption effortless. Decrypts and logs a process's SSL/TLS traffic on all major platforms. Further it allows the SSL/TLS key extraction.",

    # The content that will be shown on your project page.
    # In this case, we're displaying whatever is there in our README.md file
    long_description=README_MD,

    # Now, we'll tell PyPI what language our README file is in.
    long_description_content_type="text/markdown",


    url="https://github.com/fkie-cad/friTap",

    author_name=get_author(),
    author_email="daniel.baier@fkie.fraunhofer.de",
    license='GPL v3',

     # include other files
    package_data={
        '': [os.path.join(here, 'friTap/_ssl_log.js'), os.path.join(here, 'friTap/_ssl_log_legacy.js'), # frida agent + frida legacy agent
        os.path.join(here, 'fritap/assets/tcpdump_binaries/tcpdump_arm64_android'), # tcpdump binarys 
        os.path.join(here, 'fritap/assets/tcpdump_binaries/tcpdump_arm32_android'),
        os.path.join(here, 'fritap/assets/tcpdump_binaries/tcpdump_x86_64_android'),
        os.path.join(here, 'fritap/assets/tcpdump_binaries/tcpdump_x86_android'), ],  
    },

    exclude_package_data={'': [os.path.join(here, 'create_standalone_release')]},


    include_package_data=True,
    python_requires='>=3.6',
    packages=find_packages(exclude=('create_legacy_agent','create_standalone_release')),
    install_requires=requirements,


    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: JavaScript",
        "Topic :: Security",
        "Topic :: Software Development :: Debuggers"
    ],

    # Keywords are tags that identify your project and help searching for it
    # This field is OPTIONAL
    keywords=["mobile", "instrumentation", "frida", "hook", "SSL decryption"],

    entry_points={
            'console_scripts': [
            'fritap=friTap.friTap:main',
        ],
    },

)

